//+------------------------------------------------------------------+
//|                           LevelStopReverse EA.mq4 |
//|                         Copyright  2009, Trading Automatics Ltd |
//|                                 http://www.tradingautomatics.com |
//+------------------------------------------------------------------+
#property copyright "Copyright  2009, Trading Automatics Ltd"
#property link      "http://www.tradingautomatics.com"

extern bool       UseATRMode = true;
extern int        NonATRStopPips = 100;
extern int        ATRPeriod = 9;
extern double     ATRMultiplier = 3.0;
extern int        ATRSmoothing = 0;
extern int        MegaTrend_Period = 144;
extern int        MegaTrend_Method = MODE_SMA;
extern double     Lots = 0.01;
extern double     LotsIncreased = 0.03;
extern int        StopBuffer = 0;
extern int        SL = 50;
extern int        TP = 50;
extern bool       UseTrailingStop = false;
extern int        TrailingStop = 30;
extern int        MagicNumber = 11;
extern int        Slippage = 3;
extern bool       BrokerECN = false;
extern bool       Broker5Digits = false;

datetime   bartime = 0;

//+------------------------------------------------------------------+
//| expert initialization function                                   |
//+------------------------------------------------------------------+
int init()
{
//----
   if(Broker5Digits)
   {
      SL *= 10;
      TP *= 10;
      Slippage *= 10;
      TrailingStop *= 10;
      StopBuffer *= 10;
   }   
 
//----
   return(0);
}

//+------------------------------------------------------------------+
//| expert deinitialization function                                 |
//+------------------------------------------------------------------+
int deinit()
{
//----
   
//----
   return(0);
}

//+------------------------------------------------------------------+
//| expert start function                                            |
//+------------------------------------------------------------------+
int start()
{
   double dStop1, dStop2, dStop3;
   double dMegaTrendUp, dMegaTrendDn;
   double dEW;
   int nOrderTicket = 0;
   int nOrderType = -1;
   int nTradesNumber = 0;
   int nLongs = 0;
   int nShorts = 0;
   int iTicket = 0;
   double dLot = Lots;
   double sl = 0, tp = 0;

   nLongs = GetLongsNumber();
   nShorts = GetShortsNumber();
   nTradesNumber = nLongs + nShorts;

   for(int x = 0; x < OrdersTotal(); x++)
   {
      OrderSelect(x, SELECT_BY_POS, MODE_TRADES);
      if(OrderSymbol() == Symbol() && OrderMagicNumber() == MagicNumber)
      {
         nOrderTicket = OrderTicket();
         nOrderType = OrderType();
         break;
      }
   }
   
   // bar counting
   if(bartime != Time[0])
   {
      dStop1 = iCustom(Symbol(), 0, "MT4-LevelStop-Reverse-v2", UseATRMode, NonATRStopPips, ATRPeriod, ATRMultiplier, ATRSmoothing, 0, 1);
      dStop2 = iCustom(Symbol(), 0, "MT4-LevelStop-Reverse-v2", UseATRMode, NonATRStopPips, ATRPeriod, ATRMultiplier, ATRSmoothing, 0, 2);
      dStop3 = iCustom(Symbol(), 0, "MT4-LevelStop-Reverse-v2", UseATRMode, NonATRStopPips, ATRPeriod, ATRMultiplier, ATRSmoothing, 0, 3);
      
      if(nLongs > 0)
      {
         if(OrderStopLoss() < dStop1)
            OrderModify(nOrderTicket, OrderOpenPrice(), dStop1, OrderTakeProfit(), 0, CLR_NONE);
      }
      if(nShorts > 0)
      {
         if(OrderStopLoss() > dStop1)
            OrderModify(nOrderTicket, OrderOpenPrice(), dStop1, OrderTakeProfit(), 0, CLR_NONE);
      }

      if(Close[1] > dStop1 && (Close[2] < dStop2 || (Close[2] == dStop2 && Close[3] < dStop3)))
      {
         if(nShorts > 0)
         {
            CloseAllShorts();
         }
         if(nLongs == 0)
         {
            dMegaTrendUp = iCustom(Symbol(), 0, "Mega trend", MegaTrend_Period, MegaTrend_Method, 0, 1);
            dEW = iCustom(Symbol(), 0, "Elliott Wave Oscillator", 0, 1);
            if(dMegaTrendUp != EMPTY_VALUE && dEW > 0)
               dLot = LotsIncreased;
            if(SL > 0)
               sl = Ask - SL*Point;
            if(TP > 0)
               tp = Ask + TP*Point;
            if(BrokerECN)
            {
               iTicket = OrderSend(Symbol(), OP_BUY, dLot, Ask, Slippage, 0, 0, "", MagicNumber, 0, Blue);
               if(iTicket > 0)
               {
                  OrderModify(iTicket, OrderOpenPrice(), sl, tp, 0, CLR_NONE);
               }
            }
            else
            {
               OrderSend(Symbol(), OP_BUY, dLot, Ask, Slippage, sl, tp, "", MagicNumber, 0, Blue);
            }
         }
      }
      if(Close[1] < dStop1 && (Close[2] > dStop2 || (Close[2] == dStop2 && Close[3] > dStop3)))
      {
         if(nLongs > 0)
         {
            CloseAllLongs();
         }
         if(nShorts == 0)
         {
            dMegaTrendDn = iCustom(Symbol(), 0, "Mega trend", MegaTrend_Period, MegaTrend_Method, 1, 1);
            dEW = iCustom(Symbol(), 0, "Elliot Wave Oscillator", 0, 1);
            if(dMegaTrendDn != EMPTY_VALUE && dEW < 0)
               dLot = LotsIncreased;

            if(SL > 0)
               sl = Bid + SL*Point;
            if(TP > 0)
               tp = Bid - TP*Point;
            if(BrokerECN)
            {
               iTicket = OrderSend(Symbol(), OP_SELL, dLot, Bid, Slippage, 0, 0, "", MagicNumber, 0, Red);
               if(iTicket > 0)
               {
                  OrderModify(iTicket, OrderOpenPrice(), sl, tp, 0, CLR_NONE);
               }
            }
            else
            {
               OrderSend(Symbol(), OP_SELL, dLot, Bid, Slippage, sl, tp, "", MagicNumber, 0, Red);
            }
         }
      }
      bartime = Time[0];
   }

   if(nTradesNumber > 0)
   {
      if(UseTrailingStop)
      {
         HandleTrailing();
      }
   }
//----
   return(0);
}

int GetLongsNumber()
{
   int n = 0;
   for(int x = 0; x < OrdersTotal(); x++)
   {
      OrderSelect(x, SELECT_BY_POS, MODE_TRADES);
      if(OrderSymbol() == Symbol() && OrderMagicNumber() == MagicNumber && OrderType() == OP_BUY)
      {
         n++;
      }
   }
   return(n);
}

int GetShortsNumber()
{
   int n = 0;
   for(int x = 0; x < OrdersTotal(); x++)
   {
      OrderSelect(x, SELECT_BY_POS, MODE_TRADES);
      if(OrderSymbol() == Symbol() && OrderMagicNumber() == MagicNumber && OrderType() == OP_SELL)
      {
         n++;
      }
   }
   return(n);
}

void CloseAllLongs()
{
   int total = OrdersTotal();
   for(int i = total - 1; i >= 0; i--)
   {
      OrderSelect(i, SELECT_BY_POS, MODE_TRADES);
      if(OrderSymbol() == Symbol() && OrderMagicNumber() == MagicNumber && OrderType() == OP_BUY)
      {
         RefreshRates();
         OrderClose(OrderTicket(), OrderLots(), Bid, Slippage, Blue);
      }
   }
} 
      
void CloseAllShorts()
{
   int total = OrdersTotal();
   for(int i = total - 1; i >= 0; i--)
   {
      OrderSelect(i, SELECT_BY_POS, MODE_TRADES);
      if(OrderSymbol() == Symbol() && OrderMagicNumber() == MagicNumber && OrderType() == OP_SELL)
      {
         RefreshRates();
         OrderClose(OrderTicket(), OrderLots(), Ask, Slippage, Red);
      }
   }
} 

void HandleTrailing()
{
   for (int i = 0; i < OrdersTotal(); i++) 
   {
      OrderSelect(i, SELECT_BY_POS, MODE_TRADES);
      if(OrderType() <= OP_SELL &&  OrderSymbol() == Symbol() && OrderMagicNumber() == MagicNumber) 
      {
         if(OrderType() == OP_BUY) 
         {
            //Trailing stop
            if(UseTrailingStop && TrailingStop > 0) 
            {                 
               if(Bid - OrderOpenPrice() > Point * TrailingStop) 
               {
                  if(OrderStopLoss() < Bid - Point * TrailingStop) 
                  {
                     OrderModify(OrderTicket(), OrderOpenPrice(), Bid - Point * TrailingStop, OrderTakeProfit(), 0, MediumSeaGreen);
                     continue;
                  }
               }
            }
         } 
         else 
         {
            //Trailing stop
            if(UseTrailingStop && TrailingStop > 0) 
            {                 
               if((OrderOpenPrice() - Ask) > (Point * TrailingStop)) 
               {
                  if((OrderStopLoss() > (Ask + Point * TrailingStop)) || (OrderStopLoss() == 0)) 
                  {
                     OrderModify(OrderTicket(), OrderOpenPrice(), Ask + Point * TrailingStop, OrderTakeProfit(), 0, DarkOrange);
                     continue;
                  }
               }
            }
         }
      }
   }
}

//+------------------------------------------------------------------+